@@ -24,14 +24,11 @@
 
 #define G_SETTINGS_ENABLE_BACKEND
 
+#include <adwaita.h>
 #include <gio/gio.h>
 #include <gio/gsettingsbackend.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#ifdef GDK_WINDOWING_X11
-#include <gdk/gdkx.h>
-#endif
-#include <libhandy-1/handy.h>
 #include <stdlib.h>
 
 #include "yelp-bookmarks.h"
@@ -65,7 +62,6 @@ static const GOptionEntry entries[] = {
 typedef struct _YelpApplicationLoad YelpApplicationLoad;
 struct _YelpApplicationLoad {
     YelpApplication *app;
-    guint32 timestamp;
     gboolean new;
     gboolean fallback_help_list;
 };
@@ -82,20 +78,15 @@ static int           yelp_application_command_line     (GApplication          *a
                                                         GApplicationCommandLine *cmdline);
 static void          application_uri_resolved          (YelpUri               *uri,
                                                         YelpApplicationLoad   *data);
-static gboolean      application_window_deleted        (YelpWindow            *window,
-                                                        GdkEvent              *event,
-                                                        YelpApplication       *app);
 GSettings *          application_get_doc_settings      (YelpApplication       *app,
                                                         const gchar           *doc_uri);
-static void          application_adjust_font           (GAction               *action,
-                                                        GVariant              *parameter,
-                                                        YelpApplication       *app);
 static void          application_set_font_sensitivity  (YelpApplication       *app);
 
 static void          bookmarks_changed                 (GSettings             *settings,
                                                         const gchar           *key,
                                                         YelpApplication       *app);
 static gboolean      window_resized                    (YelpWindow            *window,
+                                                        GParamSpec *pspec,
                                                         YelpApplication       *app);
 
 typedef struct _YelpApplicationPrivate YelpApplicationPrivate;
@@ -106,17 +97,60 @@ struct _YelpApplicationPrivate {
     GPropertyAction  *show_cursor_action;
     GSimpleAction    *larger_text_action;
     GSimpleAction    *smaller_text_action;
+    GSimpleAction    *reset_text_size_action;
 
     GSettingsBackend *backend;
     GSettings *gsettings;
     GHashTable *docsettings;
 };
 
-G_DEFINE_TYPE_WITH_CODE (YelpApplication, yelp_application, GTK_TYPE_APPLICATION,
+G_DEFINE_TYPE_WITH_CODE (YelpApplication, yelp_application, ADW_TYPE_APPLICATION,
                          G_IMPLEMENT_INTERFACE (YELP_TYPE_BOOKMARKS,
                                                 yelp_application_iface_init)
                          G_ADD_PRIVATE (YelpApplication) )
 
+static void
+larger_text_activated (GSimpleAction *action,
+                       GVariant      *parameter,
+                       gpointer       data)
+{
+    YelpApplication *app = data;
+    YelpSettings *settings = yelp_settings_get_default ();
+
+    double zoom_level = yelp_settings_get_zoom_level (settings);
+
+    yelp_settings_set_zoom_level (settings, zoom_level + 0.1);
+
+    application_set_font_sensitivity (app);
+}
+
+static void
+smaller_text_activated (GSimpleAction *action,
+                        GVariant      *parameter,
+                        gpointer       data)
+{
+    YelpApplication *app = data;
+    YelpSettings *settings = yelp_settings_get_default ();
+
+    double zoom_level = yelp_settings_get_zoom_level (settings);
+
+    yelp_settings_set_zoom_level (settings, zoom_level - 0.1);
+
+    application_set_font_sensitivity (app);
+}
+
+static void
+reset_text_size_activated (GSimpleAction *action,
+                           GVariant      *parameter,
+                           gpointer       data)
+{
+    YelpApplication *app = data;
+    YelpApplicationPrivate *priv = yelp_application_get_instance_private (app);
+
+    g_settings_set_double (priv->gsettings, "zoom-level", 1.0);
+    application_set_font_sensitivity (app);
+}
+
 static void
 yelp_application_init (YelpApplication *app)
 {
@@ -204,6 +238,11 @@ yelp_application_dispose (GObject *object)
         priv->smaller_text_action = NULL;
     }
 
+    if (priv->reset_text_size_action) {
+        g_object_unref (priv->reset_text_size_action);
+        priv->reset_text_size_action = NULL;
+    }
+
     if (priv->gsettings) {
         g_object_unref (priv->gsettings);
         priv->gsettings = NULL;
@@ -234,7 +273,6 @@ yelp_application_cmdline (GApplication     *app,
     gint i;
 
     context = g_option_context_new (NULL);
-    g_option_context_add_group (context, gtk_get_option_group (FALSE));
     g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
     g_option_context_parse (context, &argc, arguments, NULL);
 
@@ -267,14 +305,11 @@ yelp_application_startup (GApplication *application)
     YelpSettings *settings;
 
     g_set_application_name (_("Help"));
+    gtk_window_set_default_icon_name ("org.gnome.Yelp");
 
     /* chain up */
     G_APPLICATION_CLASS (yelp_application_parent_class)->startup (application);
 
-    hdy_init ();
-    hdy_style_manager_set_color_scheme (hdy_style_manager_get_default (),
-                                        HDY_COLOR_SCHEME_PREFER_LIGHT);
-
     settings = yelp_settings_get_default ();
     if (editor_mode)
         yelp_settings_set_editor_mode (settings, TRUE);
@@ -282,6 +317,7 @@ yelp_application_startup (GApplication *application)
                                                        g_str_equal,
                                                        g_free,
                                                        NULL);
+
     /* Use a config file for per-document settings, because
        Ryan asked me to. */
     keyfile = g_build_filename (g_get_user_config_dir (), "yelp", "yelp.cfg", NULL);
@@ -298,60 +334,48 @@ yelp_application_startup (GApplication *application)
                                                       settings, "show-text-cursor");
     g_action_map_add_action (G_ACTION_MAP (app), G_ACTION (priv->show_cursor_action));
 
-    g_settings_bind (priv->gsettings, "font-adjustment",
-                     settings, "font-adjustment",
+    g_settings_bind (priv->gsettings, "zoom-level",
+                     settings, "zoom-level",
                      G_SETTINGS_BIND_DEFAULT);
 
     priv->larger_text_action = g_simple_action_new ("yelp-application-larger-text", NULL);
     g_signal_connect (priv->larger_text_action,
                       "activate",
-                      G_CALLBACK (application_adjust_font),
+                      G_CALLBACK (larger_text_activated),
                       app);
     g_action_map_add_action (G_ACTION_MAP (app), G_ACTION (priv->larger_text_action));
 
     priv->smaller_text_action = g_simple_action_new ("yelp-application-smaller-text", NULL);
     g_signal_connect (priv->smaller_text_action,
                       "activate",
-                      G_CALLBACK (application_adjust_font),
+                      G_CALLBACK (smaller_text_activated),
                       app);
     g_action_map_add_action (G_ACTION_MAP (app), G_ACTION (priv->smaller_text_action));
 
+    priv->reset_text_size_action = g_simple_action_new ("yelp-application-reset-text-size", NULL);
+    g_signal_connect (priv->reset_text_size_action,
+                      "activate",
+                      G_CALLBACK (reset_text_size_activated),
+                      app);
+    g_action_map_add_action (G_ACTION_MAP (app), G_ACTION (priv->reset_text_size_action));
+
     application_set_font_sensitivity (app);
 }
 
 /******************************************************************************/
 
-static void
-application_adjust_font (GAction         *action,
-                         GVariant        *parameter,
-                         YelpApplication *app)
-{
-    YelpApplicationPrivate *priv = yelp_application_get_instance_private (app);
-    gint adjustment = g_settings_get_int (priv->gsettings, "font-adjustment");
-    gint adjust = g_str_equal (g_action_get_name (action), "yelp-application-larger-text") ? 1 : -1;
-
-    adjustment += adjust;
-    g_settings_set_int (priv->gsettings, "font-adjustment", adjustment);
-
-    application_set_font_sensitivity (app);
-}
-
 static void
 application_set_font_sensitivity (YelpApplication *app)
 {
     YelpApplicationPrivate *priv = yelp_application_get_instance_private (app);
     YelpSettings *settings = yelp_settings_get_default ();
-    GParamSpec *spec = g_object_class_find_property ((GObjectClass *) YELP_SETTINGS_GET_CLASS (settings),
-                                                     "font-adjustment");
-    gint adjustment = g_settings_get_int (priv->gsettings, "font-adjustment");
-    if (!G_PARAM_SPEC_INT (spec)) {
-        g_warning ("Expcected integer param spec for font-adjustment");
-        return;
-    }
-    g_simple_action_set_enabled (priv->larger_text_action, 
-                                 adjustment < ((GParamSpecInt *) spec)->maximum);
-    g_simple_action_set_enabled (priv->smaller_text_action, 
-                                 adjustment > ((GParamSpecInt *) spec)->minimum);
+    double zoom_level = yelp_settings_get_zoom_level (settings);
+
+    g_simple_action_set_enabled (priv->larger_text_action,
+                                 zoom_level < YELP_ZOOM_LEVEL_MAX - DBL_EPSILON);
+
+    g_simple_action_set_enabled (priv->smaller_text_action,
+                                 zoom_level > YELP_ZOOM_LEVEL_MIN + DBL_EPSILON);
 }
 
 /******************************************************************************/
@@ -381,6 +405,7 @@ yelp_application_new (void)
                         "application-id", yelp,
                         "flags", G_APPLICATION_HANDLES_COMMAND_LINE,
                         "inactivity-timeout", 5000,
+                        "resource-base-path", "/org/gnome/yelp",
                         NULL);
     g_free (app_id);
     g_free (yelp);
@@ -398,7 +423,6 @@ open_uri (YelpApplication *app,
     YelpApplicationLoad *data;
     data = g_new (YelpApplicationLoad, 1);
     data->app = app;
-    data->timestamp = gtk_get_current_event_time ();
     data->new = new_window;
     data->fallback_help_list = fallback_help_list;
 
@@ -459,7 +483,6 @@ application_uri_resolved (YelpUri             *uri,
 {
     YelpWindow *window;
     gchar *doc_uri;
-    GdkWindow *gdk_window;
     YelpApplicationPrivate *priv = yelp_application_get_instance_private (data->app);
     GFile *gfile;
 
@@ -488,11 +511,11 @@ application_uri_resolved (YelpUri             *uri,
     if (window == NULL) {
         gint width, height;
         GSettings *settings = application_get_doc_settings (data->app, doc_uri);
+        YelpSettings *yelp_settings = yelp_settings_get_default ();
 
         g_settings_get (settings, "geometry", "(ii)", &width, &height);
         window = yelp_window_new (data->app);
         gtk_window_set_default_size (GTK_WINDOW (window), width, height);
-        g_signal_connect (window, "resized", G_CALLBACK (window_resized), data->app);
         priv->windows = g_slist_prepend (priv->windows, window);
 
         if (!data->new) {
@@ -503,10 +526,14 @@ application_uri_resolved (YelpUri             *uri,
             g_free (doc_uri);
         }
 
-        g_signal_connect (window, "delete-event",
-                          G_CALLBACK (application_window_deleted), data->app);
+        g_signal_connect (window, "notify::default-width", G_CALLBACK (window_resized), data->app);
+        g_signal_connect (window, "notify::default-height", G_CALLBACK (window_resized), data->app);
+
         gtk_window_set_application (GTK_WINDOW (window),
                                     GTK_APPLICATION (data->app));
+
+        /* We need to set this here, otherwise window font doesn't get initialized correctly */
+        yelp_settings_set_gtk_settings (yelp_settings, gtk_settings_get_default ());
     }
     else {
         g_free (doc_uri);
@@ -514,48 +541,22 @@ application_uri_resolved (YelpUri             *uri,
 
     yelp_window_load_uri (window, uri);
 
-    gtk_widget_show_all (GTK_WIDGET (window));
-
-    /* Metacity no longer does anything useful with gtk_window_present */
-    gdk_window = gtk_widget_get_window (GTK_WIDGET (window));
-
-#ifdef GDK_WINDOWING_X11
-    if (GDK_IS_X11_WINDOW (gdk_window)){
-        if (gdk_window)
-            gdk_x11_window_move_to_current_desktop (gdk_window);
-
-        /* Ensure we actually present the window when invoked from the command
-         * line. This is somewhat evil, but the minor evil of Yelp stealing
-         * focus (after you requested it) is outweighed for me by the major
-         * evil of no help window appearing when you click Help.
-         */
-        if (data->timestamp == 0)
-            data->timestamp = gdk_x11_get_server_time (gtk_widget_get_window (GTK_WIDGET (window)));
-
-        gtk_window_present_with_time (GTK_WINDOW (window), data->timestamp);
-    }
-    else
-#endif
-        gtk_window_present (GTK_WINDOW (window));
+    gtk_window_present (GTK_WINDOW (window));
 
     g_object_unref (uri);
     g_free (data);
 }
 
-static gboolean
-application_window_deleted (YelpWindow      *window,
-                            GdkEvent        *event,
-                            YelpApplication *app)
+void
+yelp_application_window_close_request (YelpApplication *app, GtkWindow *window)
 {
     gchar *doc_uri; /* owned by windows_by_document */
     YelpApplicationPrivate *priv = yelp_application_get_instance_private (app);
 
-    priv->windows = g_slist_remove (priv->windows, window);
+    priv->windows = g_slist_remove (priv->windows, YELP_WINDOW (window));
     doc_uri = g_object_get_data (G_OBJECT (window), "doc_uri");
     if (doc_uri)
         g_hash_table_remove (priv->windows_by_document, doc_uri);
-
-    return FALSE;
 }
 
 GSettings *
@@ -624,6 +625,7 @@ yelp_application_add_bookmark (YelpBookmarks     *bookmarks,
             g_variant_builder_add (&builder, "(sss)", page_id, icon, title);
             value = g_variant_builder_end (&builder);
             g_settings_set_value (settings, "bookmarks", value);
+            g_signal_emit_by_name (bookmarks, "bookmark-added", doc_uri, page_id);
         }
     }
 }
@@ -654,6 +656,7 @@ yelp_application_remove_bookmark (YelpBookmarks     *bookmarks,
         g_variant_iter_free (iter);
 
         g_settings_set_value (settings, "bookmarks", g_variant_builder_end (&builder));
+        g_signal_emit_by_name (bookmarks, "bookmark-removed", doc_uri, page_id);
     }
 }
 
@@ -757,9 +760,9 @@ bookmarks_changed (GSettings       *settings,
 
 static gboolean
 window_resized (YelpWindow        *window,
+                GParamSpec        *pspec,
                 YelpApplication   *app)
 {
-    YelpApplicationPrivate *priv = yelp_application_get_instance_private (app);
     YelpUri *uri;
     gchar *doc_uri;
     GSettings *settings;
@@ -772,7 +775,7 @@ window_resized (YelpWindow        *window,
         g_object_unref (uri);
         return FALSE;
     }
-    settings = g_hash_table_lookup (priv->docsettings, doc_uri);
+    settings = application_get_doc_settings (app, doc_uri);
 
     if (settings) {
         gint width, height;
